/*
	TUIO C++ Server Demo - part of the reacTIVision project
	http://reactivision.sourceforge.net/

	Copyright (c) 2005-2009 Martin Kaltenbrunner <mkalten@iua.upf.edu>

        Modified for TuioTablet: Thomas Perl <thpinfo.com>; 2009-08-03

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "SimpleSimulator.h"

#include <SDL/SDL_syswm.h>
#include <SDL/SDL_image.h>
#include <X11/Xutil.h>

#include "logo.xpm"

void SimpleSimulator::drawFrame() {
	
	if(!running) return;
        SDL_FillRect(window, &window_rect, color_bg);
        if(logo) {
            SDL_BlitSurface(logo, &(update_rects[RECT_LOGO]),
			    window, &(update_rects[RECT_LOGO]));
        }
	/*char id[3];*/
	
	// draw the cursors
	std::list<TuioCursor*> cursorList = tuioServer->getTuioCursors();
	for (std::list<TuioCursor*>::iterator tuioCursor = cursorList.begin(); tuioCursor!=cursorList.end(); tuioCursor++) {
		std::list<TuioPoint> path = (*tuioCursor)->getPath();
		if (path.size()>0) {
			
			TuioPoint last_point = path.front();
			/*glBegin(GL_LINES);
			glColor3f(0.0, 0.0, 1.0);*/
			
			for (std::list<TuioPoint>::iterator point = path.begin(); point!=path.end(); point++) {
				/*glVertex3f(last_point.getX()*width, last_point.getY()*height, 0.0f);
				glVertex3f(point->getX()*width, point->getY()*height, 0.0f);*/
				last_point.update(point->getX(),point->getY());
			}
			/*glEnd();*/
			
			// draw the finger tip
			update_rects[RECT_PREVIOUS].x = update_rects[RECT_CURRENT].x;
			update_rects[RECT_PREVIOUS].y = update_rects[RECT_CURRENT].y;

                        update_rects[RECT_CURRENT].x = (Sint16)(last_point.getX()*width)-2;
                        update_rects[RECT_CURRENT].y = (Sint16)(last_point.getY()*height)-2;
			if (update_rects[RECT_CURRENT].x < 0) update_rects[RECT_CURRENT].x = 0;
			if (update_rects[RECT_CURRENT].y < 0) update_rects[RECT_CURRENT].y = 0;
			if (update_rects[RECT_CURRENT].x + update_rects[RECT_CURRENT].w >= width) {
				update_rects[RECT_CURRENT].x -= (update_rects[RECT_CURRENT].x+update_rects[RECT_CURRENT].w-width+1);
			}
			if (update_rects[RECT_CURRENT].y + update_rects[RECT_CURRENT].h >= height) {
				update_rects[RECT_CURRENT].y -= (update_rects[RECT_CURRENT].y+update_rects[RECT_CURRENT].h-height+1);
			}

			SDL_FillRect(window, &(update_rects[RECT_CURRENT]), color_fg);
			/*glColor3f(0.75, 0.75, 0.75);
			glPushMatrix();
			glTranslatef(last_point.getX()*width, last_point.getY()*height, 0.0);
			glBegin(GL_TRIANGLE_FAN);
			for(double a = 0.0f; a <= 2*M_PI; a += 0.2f) {
				glVertex2d(cos(a) * height/100.0f, sin(a) * height/100.0f);
			}
			glEnd();
			glPopMatrix();
			
			glColor3f(0.0, 0.0, 0.0);
			glRasterPos2f((*tuioCursor)->getScreenX(width),(*tuioCursor)->getScreenY(height));
			sprintf(id,"%d",(*tuioCursor)->getCursorID());
			drawString(id);*/
		}
	}
	
	//SDL_GL_SwapBuffers();
	SDL_UpdateRects(window, 3, update_rects);
	SDL_FillRect(window, &(update_rects[RECT_CURRENT]), color_bg);
}

void SimpleSimulator::drawString(char *str) {
	/*if (str && strlen(str)) {
		while (*str) {
			glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, *str);
			str++;
		}
	}*/
}

void SimpleSimulator::toggleFullscreen() {

        if( !window || SDL_WM_ToggleFullScreen(window)!=1 ) {
                std::cout << "Unable to toggle fullscreen: " << SDL_GetError() << std::endl;
        } else {
                fullscreen = !fullscreen;
                SDL_ShowCursor(!fullscreen);
        }
}

void SimpleSimulator::processEvents()
{
    SDL_Event event;

    while( SDL_PollEvent( &event ) ) {

        switch( event.type ) {
		case SDL_KEYDOWN:
			if( event.key.keysym.sym == SDLK_ESCAPE ){
				running = false;
				SDL_ShowCursor(true);
				SDL_Quit();
			} else if( event.key.keysym.sym == SDLK_F6 ){
				toggleFullscreen();
			} else if( event.key.keysym.sym == SDLK_v ){
				verbose = !verbose;	
				tuioServer->setVerbose(verbose);
			} 
			break;
				
		case SDL_MOUSEMOTION:
			if (event.button.button & SDL_BUTTON(1)) mouseDragged((float)event.button.x/width, (float)event.button.y/height);
			break;
		case SDL_MOUSEBUTTONDOWN:
			if (event.button.button & SDL_BUTTON(1)) mousePressed((float)event.button.x/width, (float)event.button.y/height);
			break;
		case SDL_MOUSEBUTTONUP:
			if (event.button.button & SDL_BUTTON(1)) mouseReleased((float)event.button.x/width, (float)event.button.y/height);
			break;
		case SDL_QUIT:
			running = false;
			SDL_ShowCursor(true);
			SDL_Quit();
			break;
        }
    }
}

void SimpleSimulator::mousePressed(float x, float y) {
	//printf("clicked %i %i\n",x,y);
	
	TuioCursor *match = NULL;
	for (std::list<TuioCursor*>::iterator tuioCursor = stickyCursorList.begin(); tuioCursor!=stickyCursorList.end(); tuioCursor++) {
		if ((*tuioCursor)->getDistance(x,y) < (5.0f/width)) {
			match = (*tuioCursor);
			break;
		}
	}
	
	Uint8 *keystate = SDL_GetKeyState(NULL);
	if ((keystate[SDLK_LSHIFT]) || (keystate[SDLK_RSHIFT]))  {
			
		if (match!=NULL) {
			stickyCursorList.remove(match);
			tuioServer->removeTuioCursor(match);
			cursor = NULL;
		} else {
			cursor = tuioServer->addTuioCursor(x,y);
			stickyCursorList.push_back(cursor);
		}
	} else {
		if (match!=NULL) {
			cursor=match;
		} else {
			cursor = tuioServer->addTuioCursor(x,y);
		}
	}	
}

void SimpleSimulator::mouseDragged(float x, float y) {
	//printf("dragged %i %i\n",x,y);
	if (cursor==NULL) return;
	if (cursor->getTuioTime()==currentTime) return;
	tuioServer->updateTuioCursor(cursor,x,y);
}

void SimpleSimulator::mouseReleased(float x, float y) {
	//printf("released\n");
	if (cursor==NULL) return;
	
	TuioCursor *match = NULL;
	std::list<TuioCursor*> cursorList = tuioServer->getTuioCursors();
	for (std::list<TuioCursor*>::iterator tuioCursor = stickyCursorList.begin(); tuioCursor!=stickyCursorList.end(); tuioCursor++) {
		if ((*tuioCursor)->getDistance(x,y) < (5.0f/width)) {
			match = (*tuioCursor);
			break;
		}
	}
	
	if (match==NULL) {
		tuioServer->removeTuioCursor(cursor);
	}
	cursor = NULL;
}

SimpleSimulator::SimpleSimulator(char* host, int port) {

	verbose = false;
	fullscreen = true;
	window_width = 640;
	window_height = 480;
	screen_width = 800;
	screen_height = 480;

        window_rect.x = 0;
        window_rect.y = 0;
        window_rect.w = screen_width;
        window_rect.h = screen_height;

	cursor = NULL;
	if ((strcmp(host,"default")==0) && (port==0)) tuioServer = new TuioServer();
	else tuioServer = new TuioServer(host, port);
	currentTime = TuioTime::getSessionTime();
	
	//tuioServer->enablePeriodicMessages();

	if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
		std::cerr << "SDL could not be initialized: " <<  SDL_GetError() << std::endl;
		SDL_Quit();
		exit(1);
	}
	
	int videoFlags = /*SDL_OPENGL | */SDL_DOUBLEBUF;
	if( fullscreen ) {
		videoFlags |= SDL_FULLSCREEN;
		SDL_ShowCursor(false);
		width = screen_width;
		height = screen_height;
	} else {
		width = window_width;
		height = window_height;
	}
	
	window = SDL_SetVideoMode(width, height, 0, videoFlags);
	if ( window == NULL ) {
		std::cerr << "Could not open window: " << SDL_GetError() << std::endl;
		SDL_Quit();
		exit(1);
	}

        color_bg = SDL_MapRGB(window->format, 0x00, 0x00, 0x00);
        color_fg = SDL_MapRGB(window->format, 0xFF, 0xFF, 0xFF);
        logo = IMG_ReadXPMFromArray(logo_xpm);
        if (logo) {
	    update_rects[RECT_LOGO].x = 0;
            update_rects[RECT_LOGO].y = 0;
            update_rects[RECT_LOGO].w = logo->w;
            update_rects[RECT_LOGO].h = logo->h;
        }
	update_rects[RECT_CURRENT].w = 
		update_rects[RECT_CURRENT].h =
		update_rects[RECT_PREVIOUS].w =
		update_rects[RECT_PREVIOUS].h = 5;
	SDL_ShowCursor(SDL_DISABLE);
        SDL_FillRect(window, &window_rect, color_bg);

        SDL_SysWMinfo info;
        SDL_VERSION(&info.version);
        if ( SDL_GetWMInfo(&info) ) {
            Display *dpy = info.info.x11.display;
            Window win;
            if (dpy) {
              win = info.info.x11.fswindow;
              if (win) XStoreName(dpy, win, "TuioTablet - Input surface");
              win = info.info.x11.wmwindow;
              if (win) XStoreName(dpy, win, "TuioTablet - Input surface");
            }
        }

	/*glClearColor(1.0f, 1.0f, 1.0f, 0.0f);
	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);	
	glLoadIdentity();
	gluOrtho2D(0, (float)width,  (float)height, 0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();*/
}

void SimpleSimulator::run() {
	running=true;
	while (running) {
		currentTime = TuioTime::getSessionTime();
		tuioServer->initFrame(currentTime);
		processEvents();
		tuioServer->stopUntouchedMovingCursors();
		tuioServer->commitFrame();
		drawFrame();
		SDL_Delay(20);
	} 
}

int main(int argc, char* argv[])
{
	if (( argc != 1) && ( argc != 3)) {
        	std::cout << "usage: SimpleSimulator [host] [port]\n";
        	return 0;
	}

        /* Maemo-specific - see http://wiki.maemo.org/Game_development */
        putenv((char*)"SDL_VIDEO_X11_WMCLASS=TuioTabletSimulator");
	
	SimpleSimulator *app;
	if( argc == 3 ) {
		app = new SimpleSimulator(argv[1],atoi(argv[2]));
	} else app = new SimpleSimulator((char*)"localhost",0);
	
	app->run();
	delete(app);

	return 0;
}


