/**
 * TuioTablet - A frontend to the Maemo version of TUIO_CPP's SimpleSimulator
 * Copyright (c) 2009-08-03 Thomas Perl <thpinfo.com>
 *
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

 *
 **/

#include <gtk/gtk.h>

typedef struct _TuioTablet {
    GtkWidget* window;
    GtkWidget* table;
    GtkWidget* label_title;
    GtkWidget* entry_host;
    GtkWidget* entry_port;
    GtkWidget* button_launch;
} TuioTablet;

static void
on_destroy(GtkWidget* widget, gpointer data)
{
    gtk_main_quit();
}

static void
on_launch(GtkWidget* widget, gpointer data)
{
    TuioTablet* app = (TuioTablet*)data;
    gchar* command_line;

    command_line = g_strdup_printf("tuiotablet-simulator %s %s",
            gtk_entry_get_text(app->entry_host),
            gtk_entry_get_text(app->entry_port));

    g_spawn_command_line_async(command_line, NULL);
    /*fprintf(stderr, "command: %s\n", command_line);*/

    on_destroy(widget, NULL);
}

int
main(int argc, char** argv)
{
    TuioTablet app;
    gtk_init(&argc, &argv);

    gtk_window_set_default_icon_from_file("/usr/share/icons/hicolor/scalable/hildon/tuiotablet.png", NULL);

    app.table = gtk_table_new(4, 2, FALSE);
    app.label_title = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(app.label_title),
            "<b>Tuio Tablet Launcher</b>\n"
            "<span size=\"small\">http://thpinfo.com/2009/mt/</span>");
    gtk_table_attach(GTK_TABLE(app.table),
            app.label_title,
            0, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 5, 5);
    gtk_table_attach(GTK_TABLE(app.table),
            gtk_label_new("Destination host:"),
            0, 1, 1, 2, GTK_FILL, GTK_FILL, 5, 5);
    app.entry_host = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(app.entry_host), "127.0.0.1");
    gtk_table_attach(GTK_TABLE(app.table),
            app.entry_host,
            1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 5, 5);
    gtk_table_attach(GTK_TABLE(app.table),
            gtk_label_new("Destination port:"),
            0, 1, 2, 3, GTK_FILL, GTK_FILL, 5, 5);
    app.entry_port = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(app.entry_port), "3333");
    gtk_table_attach(GTK_TABLE(app.table),
            app.entry_port,
            1, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_FILL, 5, 5);
    app.button_launch = gtk_button_new_with_label("Start Tuio Tablet");
    gtk_table_attach(GTK_TABLE(app.table),
            app.button_launch,
            0, 2, 3, 4, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 5, 5);

    app.window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(app.window), "Tuio Tablet");
    gtk_container_add(GTK_CONTAINER(app.window), app.table);
    gtk_widget_show_all(app.window);

    g_signal_connect(G_OBJECT(app.button_launch), "clicked",
            G_CALLBACK(on_launch), &app);

    g_signal_connect(G_OBJECT(app.window), "destroy",
            G_CALLBACK(on_destroy), NULL);

    gtk_main();

    return 0;
}

